use crate::bussy::{channels, Command, Message};
use crate::noose::user::User;
use crate::usernames::dto::{Account, UserQuery};
use crate::utils::error::Error;
use crate::utils::structs::Context;
use serde_json::json;
use warp::{Rejection, Reply};

pub async fn get_account(
    // account: Result<AccountPubkey, Error>,
    account: Account,
    context: Context,
) -> Result<impl Reply, Rejection> {
    let mut subscriber = context.pubsub.subscribe(channels::MSG_NIP05).await;

    let command = Command::DbReqGetAccount(account.pubkey);

    context
        .pubsub
        .publish(
            channels::MSG_NOOSE,
            Message {
                source: channels::MSG_NIP05,
                content: command,
            },
        )
        .await;

    if let Ok(result) = subscriber.recv().await {
        match result.content {
            Command::DbResAccount => Ok(warp::reply::with_status(
                "ACCOUNT CREATED",
                warp::http::StatusCode::CREATED,
            )),
            Command::ServiceError(e) => Err(warp::reject::custom(e)),
            _ => Err(warp::reject::custom(Error::internal_with_message(
                "Unhandled message type",
            ))),
        }
    } else {
        Err(warp::reject::custom(Error::internal_with_message(
            "Unhandeled message type",
        )))
    }
}

pub async fn get_user(user_query: UserQuery, context: Context) -> Result<impl Reply, Rejection> {
    let name = user_query.user;
    let mut subscriber = context.pubsub.subscribe(channels::MSG_NIP05).await;

    let user = User {
        name: Some(name),
        pubkey: None,
    };
    let command = Command::DbReqGetUser(user);
    context
        .pubsub
        .publish(
            channels::MSG_NOOSE,
            Message {
                source: channels::MSG_NIP05,
                content: command,
            },
        )
        .await;

    if let Ok(message) = subscriber.recv().await {
        let mut response = json!({"names": {}, "relays": {}});
        match message.content {
            Command::DbResUser(user) => {
                response = json!({
                    "names": {
                        user.username: user.pubkey
                    },
                    "relays": {}
                });
                Ok(warp::reply::json(&response))
            }
            Command::ServiceError(e) => Ok(warp::reply::json(&response)),
            _ => Err(warp::reject::custom(Error::internal_with_message(
                "Unhandeled message type",
            ))),
        }
    } else {
        Err(warp::reject::custom(Error::internal_with_message(
            "Unhandeled message type",
        )))
    }
}
